@@ -37,8 +37,7 @@ enum
 enum
 {
   THEME_FLAG_DEFAULT = 1 << 0,
-  THEME_FLAG_CUSTOM  = 1 << 1,
-  THEME_FLAG_INITIAL  = 1 << 2
+  THEME_FLAG_CUSTOM  = 1 << 1
 };
 
 extern GtkTargetEntry drop_types[];
@@ -78,4 +77,5 @@ void      gnome_theme_manager_drag_data_received_cb     (GtkWidget        *widge
 
 
 
+
 #endif /* __GNOME_THEME_MANAGER_H__ */
